@@ -12,7 +12,7 @@ use Exporter 5.57 qw/import/;
 
 our @EXPORT_OK = qw/query_handle get_layers/;
 
-our $VERSION = '0.006';
+our $VERSION = '0.007';
 
 XSLoader::load(__PACKAGE__, $VERSION);
 
@@ -29,8 +29,10 @@ sub _flag_names {
 sub _has_flags {
 	my $check_flag = _names_to_flags(@_);
 	return sub {
-		my $iterator = shift;
+		my ($fh, $layer) = @_;
+		my $iterator = natatime(3, PerlIO::get_layers($fh, details => 1));
 		while (my ($name, $arguments, $flags) = $iterator->()) {
+			next if defined $layer and $name ne $layer;
 			my $entry = $flags & $check_flag;
 			return 1 if $entry;
 		}
@@ -38,55 +40,68 @@ sub _has_flags {
 	}
 }
 
-sub _lack_flags {
-	my @args = @_;
-	my $func = _has_flags(@args);
+our %KIND_FOR;
+sub _is_kind {
+	my $kind = shift;
 	return sub {
-		return not $func->(@_);
-	}
+		my $fh = shift;
+		my $kinds = _get_kinds($fh);
+		if (@_) {
+			my $layer = shift;
+			return exists $kinds->{$layer} && $kinds->{$layer} & $KIND_FOR{$kind} ? 1 : 0;
+		}
+		else {
+			return (grep { $kinds->{$_} & $KIND_FOR{$kind} } keys %{$kinds}) ? 1 : 0;
+		}
+	};
 }
 
-my %is_binary = map { ( $_ => 1) } qw/unix stdio perlio crlf flock creat excl/;
+my %is_binary = map { ( $_ => 1) } qw/unix stdio perlio crlf flock creat excl mmap/;
 
 my $nonbinary_flags = _names_to_flags('UTF8', 'CRLF');
 my $crlf_flags      = _names_to_flags('CRLF');
 
-my %query_for = (
+my %layer_query_for = (
 	writeable => _has_flags('CANWRITE'),
 	readable  => _has_flags('CANREAD'),
-	buffered  => _lack_flags('UNBUF'),
 	open      => _has_flags('OPEN'),
 	temp      => _has_flags('TEMP'),
 	crlf      => _has_flags('CRLF'),
 	utf8      => _has_flags('UTF8'),
 	binary    => sub {
-		my $iterator = shift;
+		my ($fh, $layer) = @_;
+		my $iterator = natatime(3, PerlIO::get_layers($fh, details => 1));
 		while (my ($name, $arguments, $flags) = $iterator->()) {
+			next if defined $layer and $name ne $layer;
 			return 0 if not $is_binary{$name} or $flags & $nonbinary_flags;
 		}
 		return 1;
 	},
-	'mappable' => sub {
-		my $iterator = shift;
+	mappable  => sub {
+		my ($fh, $layer) = @_;
+		my $iterator = natatime(3, PerlIO::get_layers($fh, details => 1));
 		while (my ($name, $arguments, $flags) = $iterator->()) {
+			next if defined $layer and $name ne $layer;
 			return 0 if not $is_binary{$name} or $flags & $crlf_flags;
 		}
 		return 1;
 	},
-	'mapped'  => sub {
-		my $iterator = shift;
+	layer     => sub {
+		my ($fh, $layer) = @_;
+		my $iterator = natatime(3, PerlIO::get_layers($fh, details => 1));
 		while (my ($name, $arguments, $flags) = $iterator->()) {
-			return 1 if $name eq 'mmap';
+			return 1 if $name eq $layer;
 		}
 		return 0;
 	},
+	buffered => _is_kind('BUFFERED'),
+	can_crlf => _is_kind('CANCRLF'),
 );
 
 sub query_handle {
 	my ($fh, $query_name, @args) = @_;
-	my $query = $query_for{$query_name} or croak "Query $query_name isn't defined";
-	my $iterator = natatime(3, PerlIO::get_layers($fh, details => 1));
-	return $query->($iterator, @args);
+	my $layer_query = $layer_query_for{$query_name} or croak "Query $query_name isn't defined";
+	return $layer_query->($fh, @args);
 }
 
 sub get_layers {
@@ -109,7 +124,7 @@ PerlIO::Layers - Querying your filehandle's capabilities
 
 =head1 VERSION
 
-Version 0.006
+Version 0.007
 
 =head1 SYNOPSIS
 
@@ -125,123 +140,63 @@ Perl's filehandles are implemented as a stack of layers, with the bottom-most us
 
 =head1 SUBROUTINES
 
-=head2 query_handle($fh, $query_name)
+=head2 query_handle($fh, $query_name [, $layer])
 
-This query a filehandle for some information. Currently supported queries include:
+This query a filehandle for some information. All queries can take an optional argument, that will test for that layer's properties instead of all layers of the handle. Currently supported queries include:
 
 =over 4
 
+=item * layer
+
+Check the presence of a certain layer. Unlike all other properties $layer is mandatory for this query.
+
 =item * utf8
 
-Check whether the filehandle handles unicode
+Check whether the filehandle/layer handles unicode
 
 =item * crlf
 
-Check whether the filehandle does crlf translation
+Check whether the filehandle/layer does crlf translation
 
 =item * binary
 
-Check whether the filehandle is binary. This test is pessimistic (for unknown layers it will assume it's not binary).
+Check whether the filehandle/layer is binary. This test is pessimistic (for unknown layers it will assume it's not binary).
 
 =item * mappable
 
-Checks whether the filehandle is memory mappable. It is the same as binary, except that the C<utf8> layer is accepted.
-
-=item * mapped
-
-Checks whether the C<mmap> layer is used.
+Checks whether the filehandle/layer is memory mappable. It is the same as binary, except that the C<utf8> layer is accepted.
 
 =item * buffered
 
-Check whether the filehandle is buffered.
+Check whether the filehandle/layer is buffered.
 
 =item * readable
 
-Check whether the filehandle is readable.
+Check whether the filehandle/layer is readable.
 
 =item * writeable
 
-Check whether the filehandle is writeable.
+Check whether the filehandle/layer is writeable.
 
 =item * open
 
-Check whether the filehandle is open.
+Check whether the filehandle/layer is open.
 
 =item * temp
 
-Check whether the filehandle refers to a temporary file.
+Check whether the filehandle/layer refers to a temporary file.
+
+=item * can_crlf
+
+Checks whether layer $argument (or any layer if $argument it not given) can do crlf translation.
 
 =back
 
 =head2 get_layers($fh)
 
-Gets information on the layers of a filehandle. It's a list with whose entries have 3 elements: the name of the layer, the arguments of the layer (may be undef) and an arrayref with the flags of the layer as strings. The flags array can contain any of these values. You probably want to use query_layers instead.
+Gets information on the layers of a filehandle. It's a list with whose entries have 3 elements: the name of the layer, the arguments of the layer (may be undef) and an arrayref with the flags of the layer as strings. The flags array can contain any of these values. You probably want to use query_layers instead.  query_handle provides a more high level interface to this, you should probably use that when you can.
 
-=over 4
-
-=item * EOF
-
-End of file has been reached.
-
-=item * CANWRITE
-
-Writes are permitted, i.e. opened as ">" or "+<" or ">>", etc.
-
-=item * CANREAD
-
-Reads are permitted i.e. opened "<" or "+>".
-
-=item * ERROR
-
-An error has occurred.
-
-=item * TRUNCATE
-
-Truncate file suggested by open mode.
-
-=item * APPEND
-
-All writes should be appends.
-
-=item * CRLF
-
-Layer is performing Win32-like "\n" mapped to CR,LF for output and CR,LF mapped to "\n" for input. Normally the provided "crlf" layer is the only layer that need bother about this. C<binmode> will mess with this flag rather than add/remove layers if the PERLIO_K_CANCRLF bit is set for the layers class.
-
-=item * UTF8
-
-Data written to this layer should be UTF-8 encoded; data provided by this layer should be considered UTF-8 encoded. Can be set on any layer by ":utf8" dummy layer. Also set on ":encoding" layer.
-
-=item * UNBUF
-
-Layer is unbuffered - i.e. write to next layer down should occur for each write to this layer.
-
-=item * WRBUF
-
-The buffer for this layer currently holds data written to it but not sent to next layer.
-
-=item * RDBUF
-
-The buffer for this layer currently holds unconsumed data read from layer below.
-
-=item * LINEBUF
-
-Layer is line buffered. Write data should be passed to next layer down whenever a "\n" is seen. Any data beyond the "\n" should then be processed.
-
-=item * TEMP
-
-File has been unlink()ed, or should be deleted on close().
-
-=item * OPEN
-
-Handle is open.
-
-=item * FASTGETS
-
-This instance of this layer supports the "fast gets " interface. Normally set based on PERLIO_K_FASTGETS for the class and by the existence of the function(s) in the table. However a class that normally provides that interface may need to avoid it on a particular instance. The "pending" layer needs to do this when it is pushed above a layer which does not support the interface.
-
-=back
-
-query_handle provides a more high level interface to this, you should probably use that when you can.
+This function is deprecated and will be removed in a future version of PerlIO::Layers.
 
 =head1 AUTHOR
 
